@@ -11,13 +11,21 @@ import subprocess  # nosec
 import uuid
 from enum import Enum
 from shutil import which
-from typing import Callable, Dict, List, Optional, Tuple, Type, TypeVar, cast
+from typing import Callable, Dict, List, Optional, Tuple, Type, TypeVar
 from uuid import UUID
 
 import pkg_resources
 import semver
 from memoization import cached
-from onefuzztypes import enums, models, primitives, requests, responses, webhooks
+from onefuzztypes import (
+    enums,
+    events,
+    models,
+    primitives,
+    requests,
+    responses,
+    webhooks,
+)
 from pydantic import BaseModel
 from six.moves import input  # workaround for static analysis
 
@@ -139,7 +147,7 @@ class Endpoint:
                 % (name, value, ",".join(values))
             )
 
-        raise Exception("%s does not expand to a value: %s" % (name, value))
+        raise Exception("Unable to find %s based on prefix: %s" % (name, value))
 
     def _disambiguate_uuid(
         self,
@@ -178,7 +186,7 @@ class Files(Endpoint):
         """ get a file from a container """
         self.logger.debug("getting file from container: %s:%s", container, filename)
         client = self._get_client(container)
-        downloaded = cast(bytes, client.download_blob(filename))
+        downloaded = client.download_blob(filename)
         return downloaded
 
     def upload_file(
@@ -291,7 +299,7 @@ class Webhooks(Endpoint):
         self,
         name: str,
         url: str,
-        event_types: List[enums.WebhookEventType],
+        event_types: List[events.EventType],
         *,
         secret_token: Optional[str] = None,
     ) -> webhooks.Webhook:
@@ -311,7 +319,7 @@ class Webhooks(Endpoint):
         *,
         name: Optional[str] = None,
         url: Optional[str] = None,
-        event_types: Optional[List[enums.WebhookEventType]] = None,
+        event_types: Optional[List[events.EventType]] = None,
         secret_token: Optional[str] = None,
     ) -> webhooks.Webhook:
         """ Update a webhook """
@@ -346,7 +354,7 @@ class Webhooks(Endpoint):
             data=requests.WebhookGet(webhook_id=webhook_id_expanded),
         )
 
-    def ping(self, webhook_id: UUID_EXPANSION) -> webhooks.WebhookEventPing:
+    def ping(self, webhook_id: UUID_EXPANSION) -> events.EventPing:
         """ ping a webhook """
 
         webhook_id_expanded = self._disambiguate_uuid(
@@ -356,7 +364,7 @@ class Webhooks(Endpoint):
         self.logger.debug("pinging webhook: %s", webhook_id_expanded)
         return self._req_model(
             "POST",
-            webhooks.WebhookEventPing,
+            events.EventPing,
             data=requests.WebhookGet(webhook_id=webhook_id_expanded),
             alternate_endpoint="webhooks/ping",
         )
@@ -777,6 +785,8 @@ class Tasks(Endpoint):
         check_asan_log: bool = False,
         check_debugger: bool = True,
         check_retry_count: Optional[int] = None,
+        check_fuzzer_help: Optional[bool] = None,
+        expect_crash_on_failure: Optional[bool] = None,
         debug: Optional[List[enums.TaskDebugFlag]] = None,
         duration: int = 24,
         ensemble_sync_delay: Optional[int] = None,
@@ -800,6 +810,8 @@ class Tasks(Endpoint):
         target_timeout: Optional[int] = None,
         target_workers: Optional[int] = None,
         vm_count: int = 1,
+        preserve_existing_outputs: bool = False,
+        colocate: bool = False,
     ) -> models.Task:
         """
         Create a task
@@ -843,6 +855,7 @@ class Tasks(Endpoint):
             pool=models.TaskPool(count=vm_count, pool_name=pool_name),
             prereq_tasks=prereq_tasks,
             tags=tags,
+            colocate=colocate,
             task=models.TaskDetails(
                 analyzer_env=analyzer_env,
                 analyzer_exe=analyzer_exe,
@@ -850,6 +863,8 @@ class Tasks(Endpoint):
                 check_asan_log=check_asan_log,
                 check_debugger=check_debugger,
                 check_retry_count=check_retry_count,
+                check_fuzzer_help=check_fuzzer_help,
+                expect_crash_on_failure=expect_crash_on_failure,
                 duration=duration,
                 ensemble_sync_delay=ensemble_sync_delay,
                 generator_exe=generator_exe,
@@ -1064,7 +1079,7 @@ class Pool(Endpoint):
     def get(self, name: str) -> models.Pool:
         self.logger.debug("get details on a specific pool")
         expanded_name = self._disambiguate(
-            "name", name, lambda x: False, lambda: [x.name for x in self.list()]
+            "pool name", name, lambda x: False, lambda: [x.name for x in self.list()]
         )
 
         return self._req_model(
@@ -1180,6 +1195,26 @@ class Node(Endpoint):
             ),
         )
 
+    def add_ssh_key(
+        self, machine_id: UUID_EXPANSION, *, public_key: str
+    ) -> responses.BoolResult:
+        self.logger.debug("add ssh public key to node: %s", machine_id)
+        machine_id_expanded = self._disambiguate_uuid(
+            "machine_id",
+            machine_id,
+            lambda: [str(x.machine_id) for x in self.list()],
+        )
+
+        return self._req_model(
+            "POST",
+            responses.BoolResult,
+            data=requests.NodeAddSshKey(
+                machine_id=machine_id_expanded,
+                public_key=public_key,
+            ),
+            alternate_endpoint="node/add_ssh_key",
+        )
+
 
 class Scaleset(Endpoint):
     """ Interact with managed scaleset pools """
@@ -1494,6 +1529,7 @@ class Onefuzz:
         client_id: Optional[str] = None,
         client_secret: Optional[str] = None,
         enable_feature: Optional[PreviewFeature] = None,
+        tenant_domain: Optional[str] = None,
     ) -> BackendConfig:
         """ Configure onefuzz CLI """
         self.logger.debug("set config")
@@ -1520,6 +1556,8 @@ class Onefuzz:
             self._backend.config.client_secret = client_secret
         if enable_feature:
             self._backend.enable_feature(enable_feature.name)
+        if tenant_domain is not None:
+            self._backend.config.tenant_domain = tenant_domain
         self._backend.app = None
         self._backend.save_config()
 
